/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.search;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import re.jpayet.mentdb.core.db.Record;
import re.jpayet.mentdb.core.db.file.experience.ExperienceManagerL1;
import re.jpayet.mentdb.core.db.file.experience.ExperienceManagerL2;
import re.jpayet.mentdb.core.entity.concentration.ConcentrationManager;
import re.jpayet.mentdb.core.entity.word.WordManager;
import re.jpayet.mentdb.core.parameter.ParameterManager;
import re.jpayet.mentdb.tools.Misc;

public class BasicSearchEngine {
	
	//Search relations
	@SuppressWarnings("unchecked")
	public static JSONArray execute(String user, String text, String typeList) throws Exception {
		
		JSONArray input_text = new JSONArray();
		
		String[] list = text.split(" ");
		
		for(int i=0;i<list.length;i++) {
			
			if (!list[i].equals("")) input_text.add(list[i]);
			
		}
		
		return execute(user, input_text, typeList);
		
	}
	
	//Search
	@SuppressWarnings("unchecked")
	public static JSONArray execute(String user, JSONArray inputVector, String typeList) throws Exception {
		
		//Initialization
		double mpa = Double.parseDouble(ParameterManager.get_value(user, "ELEVATION_MAX_%_ACCEPTANCE"));
		int concentration = ConcentrationManager.getConcentrationDepth("C[basic-search]");
		if (typeList.equals("*")) typeList = DeepSearchEngine.allTypeList;
		
		for(int z=0;z<typeList.length();z++) {
			if (DeepSearchEngine.allTypeList.indexOf(""+typeList.charAt(z))==-1) {
				
				throw new Exception("Sorry, the type '"+typeList.charAt(z)+"' is not valid ("+DeepSearchEngine.allTypeList+").");
				
			}
		}
		
		//Get all thoughts
		HashMap<String, HashMap<String, Vector<String>>> allThoughts = new HashMap<String, HashMap<String, Vector<String>>>();
		Vector<HashMap<String, Integer>> allThoughtString = new Vector<HashMap<String, Integer>>();
		for(int i=0;i<inputVector.size();i++) {
			allThoughtString.add(getAllThoughts(user, inputVector, i, inputVector.get(i)+"", allThoughts, mpa));
        }
		
		HashMap<String, Integer> alreadyUsed = new HashMap<String, Integer>();
		
		//Parse all words
		for(int j=0;j<inputVector.size();j++) {
			
			String wordName = inputVector.get(j)+"";
			
			HashMap<String, Vector<String>> allThoughtsL = allThoughts.get(wordName);
			
			//Parse all languages
			for(Entry<String, Vector<String>> e: allThoughtsL.entrySet()) {
	
				Vector<String> vectorOfThoughts = e.getValue();
								
				for(int i=0;i<vectorOfThoughts.size();i++) {
					
					String thoughtId = vectorOfThoughts.get(i);
					
					//Search relation
					for(int z=0;z<typeList.length();z++) {
						searchFromThought(user, thoughtId, ""+typeList.charAt(z), 1, alreadyUsed, allThoughtString, concentration);
					}
					
				}
								
			}
			
		}
		
		List<Map.Entry<String, Integer>> list = new LinkedList<>( alreadyUsed.entrySet() );
		Collections.sort( list, new Comparator<Map.Entry<String, Integer>>() {
			@Override
			public int compare( Map.Entry<String, Integer> o1, Map.Entry<String, Integer> o2 )
			{
				return ( o2.getValue() ).compareTo( o1.getValue() );
			}
		} );
		
		JSONArray r = new JSONArray();
		for (Map.Entry<String, Integer> entry : list) {
			
			r.add(entry.getKey()+" "+entry.getValue());
			
		}
		
		return r;
		
	}
	
	//Get all thoughts for a basic search
	@SuppressWarnings("unchecked")
	public static HashMap<String, Integer> getAllThoughts(String user, JSONArray inputVector, int index, String wordName, HashMap<String, HashMap<String, Vector<String>>> allThoughts, double mpa) throws Exception {
		
		JSONArray languages = new JSONArray();
		HashMap<String, Integer> allThoughtString = new HashMap<String, Integer>();
		
		try {
			languages = WordManager.showLanguages(user, wordName);
		} catch (Exception e) {};
		
		if (languages.size()==0 || (languages.size()==1 && (languages.get(0)+"").equals("io"))) {
			
			languages = new JSONArray();
			
			//Levenshtein distance
			JSONArray a = WordManager.searchLevenshteinDistance(user, wordName, ConcentrationManager.getConcentrationDepth("C[symbol]"), 1);
			
			if (a.size()>0) {
				
				JSONObject o = (JSONObject) a.get(0);
				String key = "";
				for(Object k : o.keySet()) {
					key = (String) k;
				}
				double v = (Double) o.get(key);
				
				//2 errors max
				if (v<=mpa) {
					wordName = key;
					inputVector.set(index, key);
					try {
						languages = WordManager.showLanguages(user, wordName);
					} catch (Exception e) {};
				}
				
			}
			
		}
		
		try {
			
			if (!allThoughts.containsKey(wordName)) {
				allThoughts.put(wordName, new HashMap<String, Vector<String>>());
			}
			
			HashMap<String, Vector<String>> allThoughtsL = allThoughts.get(wordName);
		
			//Make a basic search for the word in all languages
			for(int i=0;i<languages.size();i++) {
				
				//Get the current language
				String curLang = ""+languages.get(i);
				
				if (!allThoughtsL.containsKey(curLang)) {
					allThoughtsL.put(curLang, new Vector<String>());
				}
				
				Vector<String> curLangThoughts = allThoughtsL.get(curLang);
				
				//Get all thoughts
				//Get the first tab link
				String thoughtId = ExperienceManagerL1.getFirstTabLink(user, "W["+wordName+"]", curLang, true);
				int depth = 0;
				
				//Parse all tab links
				while (thoughtId!=null) {
					
					curLangThoughts.add(thoughtId);
					allThoughtString.put(thoughtId, depth);
					
					//Get the next tab link
					thoughtId = ExperienceManagerL1.getDownTabLink(user, "W["+wordName+"]", thoughtId, curLang, true);
					
					depth++;
				
				}
				
			}
			
		} catch (Exception e) {};
		
		return allThoughtString;
		
	}
	
	//Search relation into a thought
	public static void searchFromThought(String user, String thoughtId, String type, int depth, HashMap<String, Integer> alreadyUsed, Vector<HashMap<String, Integer>> allThoughtString, int concentration) throws Exception {
		
		//Get the first
		String tl = ExperienceManagerL2.getFirstTabLink(user, thoughtId, type);
		
		while(tl!=null && depth<=concentration) {
			
			//Get the relation id into a thought
			String relationId = Misc.atom(tl, 1, " ").substring(1);
			
			JSONObject obj = Record.getNode(relationId);
			
			String rlth = (obj.get("rlth")+"");

			int calcul = cognitiveDistance(rlth, allThoughtString);
			
			if (!alreadyUsed.containsKey(relationId)) {
				alreadyUsed.put(relationId, calcul);
			}
			
			//Get the next down node
			tl = ExperienceManagerL2.getDownTabLink(user, thoughtId, tl, type);
			
			depth++;
			
		}
		
	}
	
	//Calculate the cognitive distance
	public static int cognitiveDistance(String rThoughts, Vector<HashMap<String, Integer>> allThoughtString) {
		
		int result = 0;
				
		//Search in the relation
		for(int j=0;j<allThoughtString.size();j++) {
			for(Object th : allThoughtString.get(j).keySet()) {
				
				if (rThoughts.indexOf(th+"")>-1) {
					
					result++;
					
				}
				
			}
			
		}
		
		return result;
		
	}

}
